import numpy as np
import os
from adari_core.plots.panel import Panel
from adari_core.plots.images import ImagePlot
from adari_core.plots.points import LinePlot
from adari_core.plots.text import TextPlot
from adari_core.data_libs.master_ifu_flat import MasterIfuFlatReport
from . import MuseReportMixin


class MuseIfuFlatReport(MuseReportMixin, MasterIfuFlatReport):
    # files_needed = {}
    center_size = 500

    def __init__(self):
        super().__init__("muse_ifu_flat")

    def parse_sof(self):
        """Returns a list of files selected from a set of frames (sof).

        If more than one file fulfils the criteria, the first file
        in the array will be selected.
        """
        for path, cat in self.inputs:
            if cat == "MASTER_FLAT" and self.files_needed["MASTER_FLAT"] is None:
                self.files_needed["MASTER_FLAT"] = path

        return [self.files_needed]

    def add_text_plot(self, panel, master_flat, ext, vspace=0.3):
        """Add to the single-extension report panel the text plots requirements."""
        px, py = 0, 0
        t1 = TextPlot(columns=1, v_space=vspace)
        fname = os.path.basename(master_flat.filename())

        col1 = (
            str(master_flat[0].header.get("INSTRUME")),
            "EXTNAME: " + ext,
            "PRO CATG: " + str(master_flat[0].header.get("HIERARCH ESO PRO CATG")),
            "FILE NAME: " + fname,
            "RAW1 NAME: "
            + str(master_flat[0].header.get("HIERARCH ESO PRO REC1 RAW1 NAME")),
        )
        t1.add_data(col1)

        panel.assign_plot(t1, px, py, xext=2)

        px += 2
        t2 = TextPlot(columns=1, v_space=vspace, xext=1)
        col2 = (
            "INS.MODE: "
            + str(master_flat[0].header.get("HIERARCH ESO INS MODE", "N/A")),
            "INS.OPTI1.NAME: "
            + str(master_flat[0].header.get("HIERARCH ESO INS OPTI1 NAME", "N/A")),
            "DET.READ.CURNAME: "
            + str(master_flat[0].header.get("HIERARCH ESO DET READ CURNAME", "N/A")),
            "DATANCOM: "
            + str(master_flat[0].header.get("HIERARCH ESO PRO DATANCOM", "N/A")),
        )
        t2.add_data(col2)
        panel.assign_plot(t2, px, py, xext=1)

    def generate_single_ext_panels(self):
        panels = {}
        for hdul_dict in self.hdus:
            master_flat_hdul = hdul_dict.get("MASTER_FLAT", None)
            if master_flat_hdul is None:
                raise RuntimeError("Must provide a MASTER_FLAT to muse_lamp_flat")
            n_detectors = (len(master_flat_hdul) - 1) // 3
            # print(f"n_detectors = {n_detectors}")
            for i in range(n_detectors):
                extname = f"CHAN{i+1:02d}.DATA"
                master_flat_det = master_flat_hdul[extname]
                if master_flat_det is None:
                    continue
                panel = self.single_panel(
                    master_flat_det,
                    cutout_clipping="sigma",
                    cutout_clipping_kwargs={"nsigma": 2},
                    cutout_limits=None,
                )
                self.add_text_plot(
                    panel=panel, master_flat=master_flat_hdul, ext=extname
                )
                setup = str(
                    master_flat_hdul[0].header.get("HIERARCH ESO INS MODE", "N/A")
                )
                panels[panel] = {
                    # "master_im": master_im.filename(),
                    # "master_im_ext": master_im_ext,
                    "report_name": "muse_master_flat_single_{}_{}".format(
                        f"CHAN{i+1:02d}", setup.lower()
                    ),
                    "report_description": f"Flat single panel",
                    "report_tags": [],
                }
        return panels

    def generate_multi_ext_panels(self, ncols=6, nrows=5):
        panels = {}

        for idx, hdu_dict in enumerate(self.hdus):
            master_kw = "MASTER_FLAT"
            # Create the panels
            h = [nrows] * nrows
            h[0] = 1
            p = Panel(ncols, nrows, height_ratios=h)
            # Text Plot
            px, py = 0, 0
            # which hdul and ext to use
            vspace = 0.3
            t1 = TextPlot(columns=1, v_space=vspace)

            col1 = (
                str(hdu_dict[master_kw][0].header.get("INSTRUME")),
                "PRO CATG: "
                + str(hdu_dict[master_kw][0].header.get("HIERARCH ESO PRO CATG")),
                "RAW1 NAME: "
                + str(
                    hdu_dict[master_kw][0].header.get("HIERARCH ESO PRO REC1 RAW1 NAME")
                ),
                "FILE NAME: " + str(os.path.basename(hdu_dict[master_kw].filename())),
            )
            t1.add_data(col1)
            p.assign_plot(t1, px, py, xext=2)

            px = px + 2
            t2 = TextPlot(columns=1, v_space=vspace, xext=1)
            col2 = (
                "INS.MODE: "
                + str(
                    hdu_dict[master_kw][0].header.get("HIERARCH ESO INS MODE", "N/A")
                ),
                "INS.OPTI1.NAME: "
                + str(
                    hdu_dict[master_kw][0].header.get(
                        "HIERARCH ESO INS OPTI1 NAME", "N/A"
                    )
                ),
                "DET.READ.CURNAME: "
                + str(
                    hdu_dict[master_kw][0].header.get(
                        "HIERARCH ESO DET READ CURNAME", "N/A"
                    )
                ),
                "DATANCOM: "
                + str(
                    hdu_dict[master_kw][0].header.get(
                        "HIERARCH ESO PRO DATANCOM", "N/A"
                    )
                ),
            )
            t2.add_data(col2)
            p.assign_plot(t2, px, py, xext=1)

            setup = str(
                hdu_dict[master_kw][0].header.get("HIERARCH ESO INS MODE", "N/A")
            )
            # Assign the position of each HDUList in the panel and the plot params
            hdu_info = []
            hdu_plot_args = []
            n_detectors = (len(hdu_dict[master_kw]) - 1) // 3
            for i in range(n_detectors):
                channel_number = i + 1
                channel = hdu_dict[master_kw][channel_number].name
                channel = "MASTER_FLAT"
                py, px = np.unravel_index(channel_number - 1, (4, 6))
                # include the extra raw containing metadata
                py += 1
                hdu_info.append(
                    (idx, channel, f"CHAN{channel_number:02d}.DATA", px, py)
                )
                # Plot metadata
                hdu_plot_args.append(dict(title=f"CHAN{channel_number:02d}"))

            p = self.multi_panel(panel=p, hdu_info=hdu_info, plot_kwargs=hdu_plot_args)

            addme = {
                "report_name": f"muse_lamp_flat_multi_{setup.lower()}",
                "report_description": f"MUSE lamp flat multi panel",
                "report_tags": [],
            }
            panels[p] = addme
        return panels

    def generate_panels(self, **kwargs):
        panels = {}
        """Create both single and multiple extension panels."""
        panels = {
            **self.generate_single_ext_panels(),
            **self.generate_multi_ext_panels(),
        }
        return panels


rep = MuseIfuFlatReport()
