import numpy as np
import os
import logging

from adari_core.plots.panel import Panel
from adari_core.plots.text import TextPlot
from adari_core.data_libs.master_dark_bias import MasterDarkBiasReport
from adari_core.utils.esodet import strip_prescan_overscan_muse

from . import MuseReportMixin

logger = logging.getLogger(__name__)


class MuseMasterBiasReport(MuseReportMixin, MasterDarkBiasReport):
    center_size = 200
    # Attributes of the multi-extension report
    image_category = "master_im"
    fov_per_panel = 24
    plot_zoom_fov = False
    panel_kwargs = dict(x=6, y=4, height_ratios=[1, 4, 4, 4, 4])
    im_clipping = "percentile"
    im_clipping_kwargs = {"percentile": 50}

    def __init__(self):
        super().__init__("muse_master_bias")

        self.data_extensions = [
            "CHAN{:02d}.DATA".format(channel) for channel in range(1, 25)
        ]

    def remove_raw_scan(self, im_hdu, **kwargs):
        return strip_prescan_overscan_muse(im_hdu, debug=False, **kwargs)

    def parse_sof(self):
        first_raw = None
        master_bias = None

        for filename, catg in self.inputs:
            if catg == "MASTER_BIAS" and master_bias is None:
                master_bias = filename
            elif catg == "BIAS" and first_raw is None:
                first_raw = filename

        # Build and return the file name list
        file_lists = []
        if first_raw is not None:
            if master_bias is not None:
                file_lists.append(
                    {
                        # "raw_im": first_raw,
                        "master_im": master_bias
                    }
                )
        return file_lists

    def generate_panels(self, **kwargs):
        panels = {}
        # Generate indidual raw-cuts
        # Generate indidual raw-cuts
        for ext in self.data_extensions:
            logger.debug(f"extension {ext}")
            new_panels = super().generate_raw_cuts_panels(
                # ext=ext,
                master_im_ext=ext,
                master_title="Master Bias",
                raw_im_ext=ext.split(".")[0],
                **kwargs,
            )
            for i, (panel, panel_descr) in enumerate(new_panels.items()):
                panel_descr["report_description"] = (
                    f"MUSE bias panel - "
                    f"{os.path.basename(panel_descr['master_im'])}, "
                    f"{panel_descr['master_im_ext']}"
                )
                master_im = self.hdus[i]["master_im"]

                # Text Plot
                px = 0
                py = 0
                # which hdul and ext to use
                vspace = 0.3
                fname = os.path.basename(str(master_im.filename()))
                t1 = TextPlot(columns=1, v_space=vspace)
                col1 = (
                    str(master_im["PRIMARY"].header.get("INSTRUME")),
                    "EXTNAME: " + str(master_im[ext].header.get("EXTNAME", "N/A")),
                    "PRO CATG: "
                    + str(master_im["PRIMARY"].header.get("HIERARCH ESO PRO CATG")),
                    "FILE NAME: " + fname,
                    "RAW1 NAME: "
                    + str(
                        master_im["PRIMARY"].header.get(
                            "HIERARCH ESO PRO REC1 RAW1 NAME"
                        )
                    ),
                )
                t1.add_data(col1)
                panel.assign_plot(t1, px, py, xext=2)

                px = px + 2
                t2 = TextPlot(columns=1, v_space=vspace, xext=1)
                col2 = "DATANCOM: " + str(
                    master_im["PRIMARY"].header.get("HIERARCH ESO PRO DATANCOM", "N/A")
                )
                t2.add_data(col2)
                panel.assign_plot(t2, px, py, xext=1)

            panels = {**panels, **new_panels}

        # Generate the multi-extension panels
        img_kw_list = [
            {
                "title": channel.strip(".DATA"),
                "v_clip": self.im_clipping,
                "v_clip_kwargs": self.im_clipping_kwargs,
            }
            for channel in self.data_extensions
        ]

        new_panels = self.generate_multi_extension_panels(img_kw_list=img_kw_list)
        for i, (panel, panel_descr) in enumerate(new_panels.items()):
            hdul = self.hdus[i][self.image_category]
            setup = str(hdul["PRIMARY"].header.get("HIERARCH ESO INS MODE", "N/A"))
            panel_descr["report_name"] = "{}_{}_multi".format(
                self.name,
                setup.lower(),
            )
            panel_descr["report_description"] = (
                f"{self.name} Panel - " f"{os.path.basename(hdul.filename())}, "
            )

            text_col = (
                # "DET.DIT1: {}".format(
                #     str(hdul["PRIMARY"].header.get("HIERARCH ESO DET DIT1", "N/A"))
                # ),
                "DATANCOM: {}".format(
                    str(hdul["PRIMARY"].header.get("HIERARCH ESO PRO DATANCOM", "N/A"))
                ),
            )
            text = TextPlot(columns=1, v_space=0.3)
            text.add_data(text_col)
            panel.assign_plot(text, 2, 0, xext=1)
        panels = {**panels, **new_panels}

        return panels


rep = MuseMasterBiasReport()
