from adari_core.data_libs.master_rawdisp import MasterRawdispReport
import os
import numpy as np
import logging
from .muse_utils import MuseSetupInfo
from adari_core.plots.cut import CutPlot
from adari_core.plots.images import ImagePlot
from adari_core.utils.utils import fetch_kw_or_default
from . import MuseReportMixin

logger = logging.getLogger(__name__)


class MuseRawdispReport(MuseReportMixin, MasterRawdispReport):
    def __init__(self):
        super().__init__("muse_rawdisp")
        # Muse raw data extensions CHAN01..24
        channels = ["CHAN{:02d}".format(i) for i in range(1, 25)]
        self.raw_extensions = {c: c for c in channels}

        self.extensions = []
        ## The content of this dictionary is always: {"SOF_TAG" : "recipe"}
        self.tasks = {
            "DARK": "dark",
            #
            "BIAS": "bias",
            #
            "FLAT": "flat_lamp",
            #
            "ARC": "wavelength",
            # FIXME: both recipes uses the same tag but the metada
            # is in principle the same
            "ARC": "line_spread_function_2",
            #
            "LINGAIN_LAMP_OFF": "linearity_and_gain",
            #
            "LINGAIN_LAMP_ON": "linearity_and_gain",
            #
            "MASK": "mask",
            #
            "SKYFLAT": "sky_flat",
            #
            "AMPL": "throughput",
            #
            "STD": "preprocess_standard",
            #
            "ASTROMETRY": "preprocess_astrometry",
            #
            "ARC_LSF": "line_spread_function_lsf",
        }
        # For each keyword recipe in self.tasks, select a number of raw frames:
        # DARK: select the first frame
        # FLAT_SKY: select the second and fourth frames.
        self.select_raw_files = {
            "DARK": [1],
            "BIAS": [1],
            "FLAT": [1],
            "ARC": [1, 6, 11],
            "ARC_2": [1],
            "LINGAIN_LAMP_OFF": [1],
            "LINGAIN_LAMP_ON": [1],
            "MASK": [1],
            "SKYFLAT": [1],
            "AMPL": [1],
            "STD": [1],
            "ASTROMETRY": [1],
            "ARC_LSF": [1],
        }

        self.task_scaling = {
            # FINE TUNE SPECIFIC REPORTS, e.g.:
            # 'wavelength': {'hist': {'bins': 50, 'v_min': -100, 'v_max':1000.}},
        }
        self.setup_info = MuseSetupInfo

    def parse_sof(self):
        # we building multiple report sets, so we append multiple reports to file_lists
        # get a list of tags
        tags = list(self.tasks.keys())
        raw_count = dict.fromkeys(tags, 0)

        file_lists = []
        for filename, catg in self.inputs:
            if catg in tags:
                if filename is not None:
                    raw_count[catg] += 1
                    if raw_count[catg] in self.select_raw_files[catg]:
                        logger.debug(catg, raw_count[catg], self.select_raw_files[catg])
                        file_lists.append({"filename": filename})
                        self.sof_tag.append(catg)
        return file_lists

    def get_extensions(self):
        """Find the data extensions required for each MUSE files.

        Description
        -----------
        After the SOF has been parsed, this method iterates over the different
        HDUS files to find which extension(s) contains the data.
        """
        new_hdus_list = []
        new_sof_tag = []
        for i, filedict in enumerate(self.hdus):
            hdul = filedict["filename"]
            all_hdu_names = [hdu.name for hdu in hdul]

            # Special case for STD
            if self.sof_tag[i] == "STD":
                if "CHAN15" in all_hdu_names:
                    new_hdus_list.append(filedict)
                    new_sof_tag.append(self.sof_tag[i])
                    self.extensions.append("CHAN15")
                continue

            for chip_name in self.raw_extensions.keys():
                if chip_name in all_hdu_names:
                    new_hdus_list.append(filedict)
                    new_sof_tag.append(self.sof_tag[i])
                    self.extensions.append(chip_name)

        self.hdus = new_hdus_list
        self.sof_tag = new_sof_tag

    def generate_panels(self, **kwargs):
        panels = {}
        self.get_extensions()
        new_panels = super().generate_panels(ext=self.extensions, **kwargs)
        for i, (panel, panel_descr) in enumerate(new_panels.items()):
            # Alter the cut pos, or remove CutPlot(s) completely,
            # depending on task name
            try:
                task_name = panel_descr["task_name"]
            except KeyError as e:
                raise RuntimeError(
                    "A report has been created by "
                    "MasterRawdispReport that did "
                    "not come back with a task name "
                    "attached!"
                )
            panel_descr["report_name"] = "muse_rawdisp_{}_{}_{}_{}".format(
                task_name,
                self.raw_extensions[self.extensions[i]],
                self.sof_tag[i].lower(),
                os.path.basename(panel_descr["filename"]),
            )
            panel_descr["report_description"] = (
                f"MUSE rawdisp panel - "
                f"{panel_descr['task_name']}, "
                f"{panel_descr['tag']}, "
                f"{os.path.basename(panel_descr['filename'])}, "
                f"{panel_descr['ext']}"
            )

            # Change the cut position in the raws to 2500
            raw_cut_row = panel.retrieve(8, 3)
            raw_cut_col = panel.retrieve(8, 4)
            cutpos = 2500  # raw_cut_x.get_data_coord(raw_plot.data.shape[0] // 2, "x")
            for cutplot in (
                raw_cut_col,
                raw_cut_row,
            ):
                for label in cutplot.get_labels():
                    cutplot.update_cut_pos(label, cutpos)
                    # print(f"CutPlot dataset {label} has had cut_pos "
                    #       f"updated to {cutpos}")
            raw_cut_row.title = f"raw row @ Y={cutpos}"
            raw_cut_col.title = f"raw column @ X={cutpos}"

        panels = {**panels, **new_panels}

        return panels


rep = MuseRawdispReport()
