from adari_core.plots.panel import Panel
from adari_core.plots.text import TextPlot
from adari_core.data_libs.master_std_star_ifu import MasterSpecphotStdReport

import os
import numpy as np

from . import MuseReportMixin


class MuseSpecphotStdReport(MuseReportMixin, MasterSpecphotStdReport):
    """MUSE standard (star) recipe report class.

    Attributes
    ----------
    - files_needed: dictionary contaning the categories needed and the file names."""

    files_needed = {"DATACUBE_STD": None, "STD_RESPONSE": None, "STD_TELLURIC": None}
    n_extensions = 1
    wavelength_unit = "Angstrom"

    def __init__(self):
        super().__init__("muse_specphot_std")

    def parse_sof(self):
        """Returns a list of files selected from a set of frames (sof).

        If more than one file fullfills the criteria, the first file
        in the array will be selected.
        """
        for filename, catg in self.inputs:
            # Check that category matches the requirement
            if catg in self.files_needed.keys():
                if self.files_needed[catg] is None:
                    self.files_needed[catg] = filename
        for file, catg in self.files_needed.items():
            if file is None:
                raise IOError("ERROR: Category {} file not found".format(catg))
        return [self.files_needed]

    def generate_panels(self, **kwargs):
        panels = {}

        for hdul_dict in self.hdus:
            panel = Panel(x=3, y=4, height_ratios=[1, 5, 5, 5])

            # Metadata plots
            vspace = 0.3
            px, py = 0, 0
            t1 = TextPlot(columns=1, v_space=vspace)
            fname = os.path.basename(hdul_dict["DATACUBE_STD"].filename())
            col1 = (
                str(hdul_dict["DATACUBE_STD"][0].header.get("INSTRUME")),
                "PRO CATG: "
                + str(hdul_dict["DATACUBE_STD"][0].header.get("HIERARCH ESO PRO CATG")),
                "FILE NAME: " + fname,
                "RAW1 NAME: "
                + str(
                    hdul_dict["DATACUBE_STD"][0].header.get(
                        "HIERARCH ESO PRO REC1 RAW1 NAME"
                    )
                ),
            )
            t1.add_data(col1)

            panel.assign_plot(t1, px, py, xext=2)

            px += 2
            t2 = TextPlot(columns=1, v_space=vspace, xext=1)
            col2 = (
                "INS.MODE: "
                + str(
                    hdul_dict["DATACUBE_STD"][0].header.get(
                        "HIERARCH ESO INS MODE", "N/A"
                    )
                ),
                "INS.OPTI1.NAME: "
                + str(
                    hdul_dict["DATACUBE_STD"][0].header.get(
                        "HIERARCH ESO INS OPTI1 NAME", "N/A"
                    )
                ),
                "DET.READ.CURNAME: "
                + str(
                    hdul_dict["DATACUBE_STD"][0].header.get(
                        "HIERARCH ESO DET READ CURNAME", "N/A"
                    )
                ),
            )
            t2.add_data(col2)
            panel.assign_plot(t2, px, py, xext=1)
            # ADD PLOTS
            # White-light std image
            star_image = self.plot_star_image(
                data=hdul_dict["DATACUBE_STD"]["white"].data, title="White-light image"
            )
            panel.assign_plot(star_image, 0, 1)

            # Extracted spectrum
            ## override the value of the wavelength array
            self.wavelength = hdul_dict["STD_TELLURIC"][1].data["lambda"]
            extracted_spec = self.plot_vs_wavelength(
                data=hdul_dict["STD_TELLURIC"][1].data["ftelluric"],
                label="Telluric spectrum",
            )
            panel.assign_plot(extracted_spec, 1, 1, xext=2)
            # Response spectrum
            ## override the value of the wavelength array
            self.wavelength = hdul_dict["STD_RESPONSE"][1].data["lambda"]
            response_spec = self.plot_vs_wavelength(
                data=hdul_dict["STD_RESPONSE"][1].data["response"],
                label="Response spectrum",
            )
            panel.assign_plot(response_spec, 0, 2, xext=3)

            response_err_spec = self.plot_vs_wavelength(
                data=hdul_dict["STD_RESPONSE"][1].data["resperr"],
                label="Response spectrum error",
            )
            panel.assign_plot(response_err_spec, 0, 3, xext=3)

            # Metadata
            setup = str(
                hdul_dict["DATACUBE_STD"][0].header.get("HIERARCH ESO INS MODE", "N/A")
            )
            panels[panel] = {
                "report_name": f"muse_standard_std_{setup}",
                "report_description": "Spectrophotometric Standard Star",
                "report_tags": [],
            }
        return panels


rep = MuseSpecphotStdReport()
