import numpy as np
import os
from adari_core.plots.panel import Panel
from adari_core.plots.images import ImagePlot
from adari_core.plots.points import LinePlot
from adari_core.plots.text import TextPlot
from adari_core.data_libs.master_wave_cal import MasterWaveCalReport
from . import MuseReportMixin


class MuseWaveCalReport(MuseReportMixin, MasterWaveCalReport):
    """
    MUSE Wavelength calibration report.

    MUSE does only require the single-extension panels.
    """

    files_needed = {
        "ARC_RED_LAMP_1": None,
        "ARC_RED_LAMP_2": None,
        "ARC_RED_LAMP_3": None,
    }
    center_size = 500

    def __init__(self, name="muse_wave_cal"):
        super().__init__(name)

    def parse_sof(self):
        """Returns a list of files selected from a set of frames (sof)."""
        arc_file_number = 0
        for path, cat in self.inputs:
            if cat == "ARC_RED_LAMP":
                arc_file_number += 1
                self.files_needed[f"ARC_RED_LAMP_{arc_file_number}"] = path

        # Throw an error if the number of files is less than expected
        if arc_file_number < 3:
            raise IOError(
                f"Expected number of input ARC_RED_LAMP files is 3, found {arc_file_number}"
            )
        return [self.files_needed]

    def add_text_plot(self, panel, master_img, extension_idx):
        """Add to the report panel the text plots required for each extension."""
        px, py = 0, 0
        vspace = 0.3
        t1 = TextPlot(columns=1, v_space=vspace)
        fname = os.path.basename(master_img.filename())

        extname = master_img[extension_idx].header.get("EXTNAME", "N/A")

        col1 = (
            master_img["PRIMARY"].header.get("INSTRUME"),
            "EXTNAME: " + extname,
            "PRO CATG: " + master_img["PRIMARY"].header.get("HIERARCH ESO PRO CATG"),
            "FILE NAME: " + fname,
            "RAW NAME: "
            + master_img["PRIMARY"].header.get(f"HIERARCH ESO PRO REC1 RAW1 NAME"),
        )
        t1.add_data(col1)

        panel.assign_plot(t1, px, py, xext=2)

        px += 2
        t2 = TextPlot(columns=1, v_space=vspace, xext=1)
        col2 = (
            "INS.MODE: "
            + str(master_img["PRIMARY"].header.get("HIERARCH ESO INS MODE", "N/A")),
            "INS.OPTI1.NAME: "
            + str(
                master_img["PRIMARY"].header.get("HIERARCH ESO INS OPTI1 NAME", "N/A")
            ),
            "DET.READ.CURNAME: "
            + str(
                master_img["PRIMARY"].header.get("HIERARCH ESO DET READ CURNAME", "N/A")
            ),
        )
        t2.add_data(col2)
        panel.assign_plot(t2, px, py, xext=1)

    def single_report_panels(self):
        """Generate the single panel report panels."""
        panels = {}
        # Extension containing the data of each ARC_LAMP file
        hdul_data_extentions = "CHAN01.DATA"
        for hdul_dict in self.hdus:
            for hdul_key, val in self.files_needed.items():
                if val is None:
                    continue
                master_img_hdul = hdul_dict[hdul_key]
                # Generate the panel
                panel = self.generate_single_panel(
                    master_img_hdul[hdul_data_extentions]
                )
                # Add the metadata to the plot
                self.add_text_plot(
                    panel=panel, master_img=master_img_hdul, extension_idx="PRIMARY"
                )

                setup = master_img_hdul["PRIMARY"].header.get(
                    "HIERARCH ESO INS MODE", "N/A"
                )
                panels[panel] = {
                    "report_name": "{}_{}_{}".format(
                        self.name,
                        hdul_key.lower(),
                        setup.lower(),
                    ),
                    "report_description": f"MUSE wavelength calibration single panel",
                    "report_tags": [],
                }

        return panels

    def generate_panels(self, **kwargs):
        panels = {**self.single_report_panels()}
        return panels


rep = MuseWaveCalReport()
