from edps import data_source
from edps.generator.time_range import *

from .muse_classification import *

# Convention for Data sources Association rule levels:
# Each data source can have several match function which correspond to different
# quality levels for the selected data. The level is specified as a number that
# follows this convention:
#   level < 0: more restrictive than the calibration plan
#   level = 0 follows the calibration plan
#   level = 1 quality sufficient for QC1 certification
#   level = 2 probably still acceptable quality
#   level = 3 significant risk of bad quality results


# Grouping rules
#   ins.mode: takes into account spectral configuration, field mode and
#             the use of the AO system
#   ins.opti2.name: takes into account only the field mode
#   ins.opti1.name: takes into account only the spectral setup, i.e.
#                   order sorting filter, notch filter
group_by_template = [kwd.tpl_start]
group_by_ins_mode = [kwd.ins_mode] + group_by_template
group_by_field_mode = [kwd.ins_opti2_name] + group_by_template
group_by_spectral_setup = [kwd.ins_opti1_name] + group_by_field_mode
group_by_target_name = [kwd.ins_mode, kwd.obs_targ_name]

# general setup and matching keywords:
mode_setup = [kwd.ins_mode, kwd.det_read_curid]
match_mode_filter_and_readout = mode_setup + [kwd.ins_opti1_name]
readout_setup = [kwd.det_read_curid]
field_and_readout_setup = [kwd.ins_opti2_name, kwd.det_read_curid]
setup_field = [kwd.ins_opti2_name]

# --- Raw files data sources -----------------------------------------------------------------------

# Raw files to measure instrument throughput
raw_throughput = (data_source()
                  .with_classification_rule(throughput_class)
                  .with_setup_keywords(mode_setup)
                  .with_grouping_keywords(group_by_template)
                  .with_min_group_size(3)
                  .build())
# Raw biases
raw_bias = (data_source()
            .with_classification_rule(bias_class)
            .with_grouping_keywords(group_by_template)
            .with_min_group_size(11)
            .with_setup_keywords(readout_setup)
            .with_match_keywords(readout_setup, time_range=RelativeTimeRange(-0.8, 0.8), level=0)
            .with_match_keywords(readout_setup, time_range=TWO_WEEKS, level=1)
            .with_match_keywords(readout_setup, time_range=UNLIMITED, level=3).build())

# Raw darks
raw_dark = (data_source()
            .with_classification_rule(dark_class)
            .with_grouping_keywords(group_by_template)
            .with_min_group_size(5)
            .with_setup_keywords(readout_setup)
            .with_match_keywords(readout_setup, time_range=ONE_MONTH, level=0)
            .with_match_keywords(readout_setup, time_range=QUARTERLY, level=1)
            .with_match_keywords(readout_setup, time_range=UNLIMITED, level=3).build())

# Raw files for monitoring detector linearity
raw_lingain = (data_source()
               .with_classification_rule(lingain_bias_class)
               .with_classification_rule(lingain_flat_class)
               .with_grouping_keywords(group_by_template)
               .with_min_group_size(54)
               .with_setup_keywords(readout_setup)
               .with_match_keywords(readout_setup, time_range=ONE_WEEK, level=0)
               .with_match_keywords(readout_setup, time_range=RelativeTimeRange(-10, 10), level=1)
               .with_match_keywords(readout_setup, time_range=UNLIMITED, level=3).build())

# -- Raw flat fields ---------------------------------------
# Lamp flats
raw_flat_lamp = (data_source()
                 .with_classification_rule(flat_lamp_class)
                 .with_grouping_keywords(group_by_template)
                 .with_min_group_size(11)
                 .with_setup_keywords(mode_setup)
                 .with_match_keywords(match_mode_filter_and_readout, time_range=RelativeTimeRange(-0.8, 0.8), level=0)
                 .with_match_keywords(match_mode_filter_and_readout, time_range=TWO_DAYS, level=1)
                 .with_match_keywords(match_mode_filter_and_readout, time_range=TWO_WEEKS, level=2)
                 .with_match_keywords(match_mode_filter_and_readout, time_range=UNLIMITED, level=3)
                 .build())
# Sky flats
raw_flat_sky = (data_source()
                .with_classification_rule(flat_sky_class)
                .with_grouping_keywords(group_by_ins_mode)
                .with_min_group_size(3)
                .with_setup_keywords(match_mode_filter_and_readout)
                .with_match_keywords(match_mode_filter_and_readout, time_range=SAME_NIGHT, level=-1)
                .with_match_keywords(match_mode_filter_and_readout, time_range=ONE_WEEK, level=0)
                .with_match_keywords(match_mode_filter_and_readout, time_range=UNLIMITED, level=3)
                .build())

# Nighttime flats
TWO_HOURS = RelativeTimeRange(-2.0 / 24.0, 2.0 / 24.0)
FOUR_HOURS = RelativeTimeRange(-4.0 / 24.0, 4.0 / 24.0)
raw_flat_illum = (data_source()
                  .with_classification_rule(flat_illum_class)
                  .with_setup_keywords(field_and_readout_setup)
                  .with_match_keywords(field_and_readout_setup + [kwd.prog_id, kwd.obs_id],
                                       time_range=TWO_HOURS, level=-1)
                  .with_match_keywords(field_and_readout_setup, time_range=TWO_HOURS, level=0)
                  .with_match_keywords(field_and_readout_setup, time_range=FOUR_HOURS, level=1)
                  .with_grouping_keywords([kwd.tpl_start])
                  .build())
# ----------------------------------------------------------

raw_mask = (data_source()
            .with_classification_rule(mask_class)
            .with_grouping_keywords(group_by_ins_mode)
            .with_min_group_size(70)
            .with_setup_keywords(mode_setup)
            .with_match_function(rules.is_associated, time_range=ONE_MONTH, level=0)
            .with_match_function(rules.is_associated, time_range=UNLIMITED, level=3)
            .build())

# -- Files for measuring the Line Spread Function and wavelenght calibration -----------------------
# Dedicated LSF exposures
raw_lsf = (data_source()
           .with_classification_rule(lsf_class)
           .with_min_group_size(45)
           .with_grouping_keywords(group_by_template)
           .with_setup_keywords(mode_setup)
           .with_match_keywords(mode_setup, time_range=QUARTERLY, level=0)
           .with_match_keywords(mode_setup, time_range=UNLIMITED, level=3)
           .build())

# Static LSF file
lsf_static = (data_source()
              .with_classification_rule(lsf_static_class)
              .with_match_keywords(mode_setup, time_range=IN_THE_PAST, level=0)
              .with_match_keywords(mode_setup, time_range=UNLIMITED, level=3)
              .build())

# Raw arcs (both for wavelenght calibration and LSF characterization)
raw_arc = (data_source()
           .with_classification_rule(arc_class)
           .with_grouping_keywords(group_by_template)
           .with_min_group_size(15)
           .with_setup_keywords(match_mode_filter_and_readout)
           .with_match_keywords(match_mode_filter_and_readout, time_range=RelativeTimeRange(-0.8, 0.8), level=0)
           .with_match_keywords(match_mode_filter_and_readout, time_range=TWO_WEEKS, level=1)
           .with_match_keywords(match_mode_filter_and_readout, time_range=UNLIMITED, level=3)
           .build())
# --------------------------------------------------------------------------------------------------

# Raw sky observations
raw_sky = (data_source()
           .with_classification_rule(sky_class)
           .with_grouping_keywords([kwd.arcfile])
           .with_setup_keywords([kwd.ins_mode, kwd.det_read_curid])
           .with_match_keywords([kwd.tpl_start], level=0)
           # .with_match_keywords([kwd.instrume], time_range=SAME_NIGHT, level=3)
           .build())

raw_sky_sci = (data_source()
               .with_classification_rule(sky_class)
               .with_grouping_keywords([kwd.arcfile])
               .with_setup_keywords([kwd.ins_mode, kwd.det_read_curid])
               .with_match_keywords([kwd.arcfile])
               .build())

# Raw standard star
raw_std = (data_source()
           .with_classification_rule(std_class)
           .with_grouping_keywords([kwd.arcfile])
           .with_setup_keywords(mode_setup)
           .with_match_keywords(mode_setup + [kwd.night], time_range=ONE_DAY, level=0)  # same night
           .with_match_keywords(mode_setup, time_range=UNLIMITED, level=3)
           .build())

# Raw data to compute astrometry
raw_astrometry = (data_source()
                  .with_classification_rule(astrometry_class)
                  .with_grouping_keywords(group_by_ins_mode)
                  .with_setup_keywords(setup_field)
                  .with_match_keywords(setup_field, time_range=ONE_MONTH, level=0)
                  .with_match_keywords(setup_field, time_range=RelativeTimeRange(-60, 60), level=1)
                  .with_match_keywords(setup_field, time_range=UNLIMITED, level=3)
                  .build())

# Raw target observations
raw_object = (data_source()
              .with_classification_rule(object_class)
              .with_grouping_keywords([kwd.arcfile])
              .with_match_keywords([kwd.arcfile])
              .with_setup_keywords([kwd.obs_targ_name, kwd.ins_mode, kwd.det_read_curid])
              .build())

# --- Data sources from static calibrations and user-provided files --------------------------------


badpix_table = (data_source()
                .with_classification_rule(badpix_table_class)
                .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                .with_match_function(rules.is_associated, time_range=UNLIMITED, level=3)
                .build())

nonlinearity_gain = (data_source()
                     .with_classification_rule(nonlinearity_gain_class)
                     .with_match_keywords(readout_setup, time_range=IN_THE_PAST, level=0)
                     .with_match_keywords(readout_setup, time_range=UNLIMITED, level=1)
                     .build())

vignetting_mask = (data_source()
                   .with_classification_rule(vignetting_mask_class)
                   .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                   .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                   .build())

line_catalog = (data_source()
                .with_classification_rule(line_catalog_class)
                .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                .build())

extinct_table = (data_source()
                 .with_classification_rule(extinct_table_class)
                 .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                 .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                 .build())

filter_list = (data_source()
               .with_classification_rule(filter_list_class)
               .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
               .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
               .build())

std_flux_table = (data_source()
                  .with_classification_rule(std_flux_table_class)
                  .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                  .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                  .build())

telluric_regions = (data_source()
                    .with_classification_rule(telluric_regions_class)
                    .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                    .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                    .build())

astrometry_reference = (data_source()
                        .with_classification_rule(astrometry_reference_class)
                        .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                        .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                        .build())

astrometry_table = (data_source()
                    .with_classification_rule(static_astrometry_wcs_class)
                    .with_match_keywords(setup_field, time_range=IN_THE_PAST, level=0)
                    .with_match_keywords(setup_field, time_range=UNLIMITED, level=1)
                    .build())

geometry_reference = (data_source()
                      .with_classification_rule(static_geometry_table)
                      .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
                      .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
                      .build())

sky_lines = (data_source()
             .with_classification_rule(sky_lines_class)
             .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=0)
             .with_match_function(rules.is_associated, time_range=UNLIMITED, level=1)
             .build())

sky_continuum = (data_source()
                 .with_classification_rule(SKY_CONTINUUM)
                 .with_match_keywords([kwd.tpl_start], level=0)
                 .build())

sky_mask = (data_source()
            .with_classification_rule(SKY_MASK)
            .with_match_keywords([kwd.tpl_start], level=0)
            .build())

raman_lines = (data_source()
               .with_classification_rule(raman_lines_class)
               .with_match_function(rules.is_associated_raman_lines, time_range=IN_THE_PAST, level=0)
               .with_match_function(rules.is_associated_raman_lines, time_range=UNLIMITED, level=1)

               .build())

output_wcs = (data_source()
              .with_classification_rule(output_wcs_class)
              .with_match_keywords([kwd.tpl_start], level=0)
              .build())

autocal_factors = (data_source()
                   .with_classification_rule(autocal_factors_class)
                   .with_match_keywords([kwd.night, kwd.ins_mode], level=0)
                   .build())

offset_list = (data_source()
               .with_classification_rule(offset_list_class)
               .with_match_keywords([kwd.tpl_start], level=0)
               .build())

source_list = (data_source()
               .with_classification_rule(source_list_class)
               .with_match_keywords([kwd.tpl_start], level=0)
               .build())
