from edps import QC1_CALIB, ReportInput, FilterMode
from edps import subworkflow, task, alternative_associated_inputs

from .muse_datasources import *
from .muse_task_functions import *


# This subworkflow creates the model of the line spread function (LSF) to be used in the data reduction.
@subworkflow("line_spread_function", "")
def lsf(bias, dark, flat_lamp, wavelength):
    # --- Task to generate Line Spread Function calibrations from arc raw frames
    line_spread_function_from_arcs = (task("line_spread_function_arc")
                                      .with_recipe("muse_lsf")
                                      .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
                                      .with_report("muse_lsf", ReportInput.RECIPE_INPUTS_OUTPUTS)
                                      .with_main_input(raw_arc)
                                      .with_associated_input(badpix_table, min_ret=0)
                                      .with_associated_input(line_catalog)
                                      .with_associated_input(bias, [MASTER_BIAS])
                                      .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                                      .with_associated_input(flat_lamp, [TRACE_TABLE])
                                      .with_associated_input(wavelength, [WAVECAL_TABLE])
                                      .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                                      .build())

    # --- Task to generate Line Spread Function calibrations from LSF raw frames
    line_spread_function_from_lsf = (task("line_spread_function_lsf")
                                     .with_recipe("muse_lsf")
                                     .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
                                     .with_report("muse_lsf", ReportInput.RECIPE_INPUTS_OUTPUTS)
                                     .with_main_input(raw_lsf)
                                     .with_associated_input(badpix_table, min_ret=0)
                                     .with_associated_input(line_catalog)
                                     .with_associated_input(bias, [MASTER_BIAS])
                                     .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                                     .with_associated_input(flat_lamp, [TRACE_TABLE])
                                     .with_associated_input(wavelength, [WAVECAL_TABLE])
                                     .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                                     .with_meta_targets([QC1_CALIB])
                                     .build())

    # The Line Spread Function calibrations to use are selected on the basis of user's choice
    # by setting the lsfmode workflo parameter in muse_parameters.yaml
    # lsfmode=any              Try to construct the LSF from dedicated lsf raw exposures. If not
    #                          available, try to use arc raw exposures. If not available, use static
    #                          calibration from the pipeline distribution.
    # lsfmode=lsf              Try to construct the LSF from dedicated lsf raw exposures. If not
    #                          available, use static calibration from the pipeline distribution.
    # lsfmode=arc              Try to construct the LSF from dedicated arc raw exposures. If not
    #                          available, use static calibration from the pipeline distribution.
    # lsfmode=static (default) Use static calibration from the pipeline distribution.

    lsf_calibrations = (alternative_associated_inputs()
                        .with_associated_input(line_spread_function_from_lsf, [LSF_PROFILE],
                                               condition=use_lsf_or_any)
                        .with_associated_input(line_spread_function_from_arcs, [LSF_PROFILE],
                                               condition=use_arc_or_any)
                        .with_associated_input(lsf_static))

    return lsf_calibrations, line_spread_function_from_lsf, line_spread_function_from_arcs
