from edps import alternative_associated_inputs, FilterMode, ReportInput, match_rules
from edps import subworkflow, task, QC1_CALIB

from .muse_datasources import *
from .muse_task_functions import *


# This sub-workflow computes the atmospheric transmission by running molecfit on telluric standard star.
@subworkflow("monitoring_and_long_term_calibs", "")
def monitoring(bias, dark, flat_lamp, wavelength):
    # --- Task to compute the linearity correction and to monitor the detector
    linearity_and_gain = (task("linearity_and_gain")
                          .with_recipe("muse_lingain")
                          .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
                          .with_main_input(raw_lingain)
                          .with_associated_input(badpix_table, min_ret=0)
                          .with_associated_input(bias, [MASTER_BIAS])
                          .with_associated_input(flat_lamp, [TRACE_TABLE])
                          .with_meta_targets([QC1_CALIB])
                          .with_input_filter(MASTER_FLAT, mode=FilterMode.REJECT)
                          .build())

    # ---Task to compute the geometry table from dedicated sky calibrations
    geometry = (task("geometry")
                .with_recipe("muse_geometry")
                .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
                .with_report("muse_geometry", ReportInput.RECIPE_INPUTS_OUTPUTS)
                .with_main_input(raw_mask)
                .with_associated_input(badpix_table, min_ret=0)
                .with_associated_input(line_catalog)
                .with_associated_input(bias, [MASTER_BIAS])
                .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                .with_associated_input(flat_lamp, [TRACE_TABLE])
                .with_associated_input(wavelength, [WAVECAL_TABLE])
                .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                .with_meta_targets([QC1_CALIB])
                .build())

    # Selection of geometry calibrations to use, depending on the value of the workflow parameter "recompute_geometry".
    # recompute_geomtrey = "yes". If geometry calibrations are present in the dataset, they are reduced and associated.
    # recompute_geomtrey = "no". Static geometry tables are used (default).

    geometry_calibrations = (alternative_associated_inputs()
                             .with_associated_input(geometry, [GEOMETRY_TABLE], condition=recompute_geometry)
                             .with_associated_input(geometry_reference))

    # Throughput in NWM cannot use geometry in WFM, therefore I create its own match rule
    match_geometry_for_throughput = (match_rules()
                                     .with_match_keywords([kwd.ins_opti2_name], time_range=IN_THE_PAST, level=0)
                                     .with_match_keywords([kwd.ins_opti2_name], time_range=UNLIMITED, level=1))

    geometry_calibrations_for_throughput = (alternative_associated_inputs()
                                            .with_associated_input(geometry, [GEOMETRY_TABLE],
                                                                   condition=recompute_geometry,
                                                                   match_rules=match_geometry_for_throughput, min_ret=0)
                                            .with_associated_input(geometry_reference,
                                                                   match_rules=match_geometry_for_throughput,
                                                                   min_ret=0))

    # Astrometric calibrations taken during the night requires geometry calibration taken 1 or 2 days after.
    match_geometry_for_astrometry_future = (match_rules()
                                            .with_match_function(rules.is_associated,
                                                                 time_range=RelativeTimeRange(0, 2),
                                                                 level=0))
    match_geometry_for_astrometry_static = (match_rules()
                                            .with_match_function(rules.is_associated, time_range=IN_THE_PAST, level=1))

    geometry_calibrations_for_astrometry = (alternative_associated_inputs()
                                            .with_associated_input(geometry, [GEOMETRY_TABLE],
                                                                   match_rules=match_geometry_for_astrometry_future)
                                            .with_associated_input(geometry_reference,
                                                                   match_rules=match_geometry_for_astrometry_static))

    # --- Task to measure the instrument throughput. Used for monitoring purposes only.
    throughput = (task("throughput")
                  .with_recipe("muse_ampl")
                  .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
                  .with_report("muse_throughput", ReportInput.RECIPE_INPUTS_OUTPUTS)
                  .with_main_input(raw_throughput)
                  .with_associated_input(filter_list)
                  .with_associated_input(badpix_table, min_ret=0)
                  .with_associated_input(bias, [MASTER_BIAS])
                  .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                  .with_associated_input(flat_lamp, [TRACE_TABLE])
                  .with_associated_input(wavelength, [WAVECAL_TABLE])
                  .with_alternative_associated_inputs(geometry_calibrations_for_throughput)
                  .with_input_filter(MASTER_FLAT, ARC_RED_LAMP, mode=FilterMode.REJECT)
                  .with_meta_targets([QC1_CALIB])
                  .build())

    return linearity_and_gain, geometry_calibrations, geometry_calibrations_for_astrometry, throughput
