from edps import QC0, SCIENCE, CALCHECKER, IDP, task, subworkflow, match_rules
from edps import alternative_associated_inputs, FilterMode, copy_upstream

from .muse_datasources import *
from .muse_task_functions import *

IDP_DEEP = "idp_deep"


# This subworkflow reduces and (eventually) combine science exposures
@subworkflow("process_and_combine_object", "")
def process_science(sky, response, lsf_calibrations, astrometry_calibrations, bias, dark, lamp_flat, wavelength,
                    geometry_calibrations, sky_flat, science_sky):
    # --- Pre-process object raw exposures

    preprocess_science = (task("preprocess_science")
                          .with_recipe("muse_scibasic")
                          .with_main_input(raw_object)
                          .with_associated_input(badpix_table, min_ret=0)
                          .with_associated_input(nonlinearity_gain, min_ret=0, condition=use_linearity_gain)
                          .with_associated_input(bias, [MASTER_BIAS])
                          .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                          .with_associated_input(lamp_flat, [TRACE_TABLE, MASTER_FLAT])
                          .with_associated_input(wavelength, [WAVECAL_TABLE])
                          .with_alternative_associated_inputs(geometry_calibrations)
                          .with_associated_input(sky_flat, [TWILIGHT_CUBE], min_ret=1, condition=use_skyflats)
                          .with_associated_input(raw_flat_illum, min_ret=0)
                          .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                          .with_meta_targets([QC0, CALCHECKER, IDP, IDP_DEEP])
                          .build())

    # Selection of sky background (SKY_LINES and SKY_CONTINUUM). If dedicated sky calibrations are found in the dataset,
    # then they are used. Otherwise static sky_lines calibratins are used.
    sky_background = (alternative_associated_inputs()
                      .with_associated_input(sky, [MASTER_SKYLINES, SKY_CONTINUUM])
                      .with_associated_input(sky_lines))

    # --- Reduction of Object pixel tables.

    # Setting the strategy for autocalibration factors.
    associate_sky_autocalibrations = (match_rules()
                                      .with_match_keywords([kwd.night, kwd.ins_mode], level=0))

    same_dataset = (match_rules()
                    .with_match_keywords([kwd.mjd_obs]))
    autocalibration_from_object_sky = (task("autocalibration_from_object_sky")
                                       .with_condition(use_autocalibration_sky)
                                       .with_function(copy_upstream)
                                       .with_main_input(preprocess_science)
                                       .with_associated_input(science_sky, match_rules=associate_sky_autocalibrations)
                                       .with_output_filter(autocal_factors_class, mode=FilterMode.SELECT)
                                       .build())

    science = (task("object")
               .with_recipe("muse_scipost")
               .with_main_input(preprocess_science)
               .with_associated_input(extinct_table)
               .with_associated_input(filter_list, min_ret=0)
               .with_associated_input(raman_lines, min_ret=0)
               .with_associated_input(offset_list, min_ret=0)
               .with_associated_input(output_wcs, min_ret=0)
               .with_associated_input(autocal_factors, min_ret=1, condition=use_autocalibration_user)
               .with_associated_input(autocalibration_from_object_sky, min_ret=0, condition=use_autocalibration_sky,
                                      match_rules=same_dataset)
               .with_associated_input(response, [STD_TELLURIC, STD_RESPONSE])
               .with_alternative_associated_inputs(sky_background)
               .with_alternative_associated_inputs(lsf_calibrations)
               .with_associated_input(sky_mask, min_ret=0)
               .with_alternative_associated_inputs(astrometry_calibrations)
               .with_grouping_keywords(['$combine_science', kwd.ins_mode])
               .with_grouping_function(variable_clustering_by_sky_position)
               .with_min_group_size(2)  # Combination of reduced exposures require at least 2 exposures
               .with_job_processing(job_object)
               .with_meta_targets([QC0, SCIENCE, CALCHECKER, IDP, IDP_DEEP])
               .build())

    # --- Alignment and combination of reduced exposures ---------------------------------------------------------------
    alignment = (task("alignment")
                 .with_recipe("muse_exp_align")
                 .with_main_input(science)
                 .with_associated_input(source_list, min_ret=0)
                 .with_input_filter(image_fov_class)
                 .with_meta_targets([IDP, IDP_DEEP])
                 .build())

    # The selection of science exposures to combine is done in two steps.
    # 1. First, exposures are clustered by their sky coordinates. This step is regulated by two workflow parameters,
    #   "max_separation" and "max_diameter" . In order to include an exposure in a group of files for combination:
    #      - the exposure should be less distant than "max_separation" from the group.
    #      - the size of the group should be less than "max_diameter" after the inclusion of the new exposure.
    #
    # 2. Exposures in each cluster are grouped according to the value of a selected header keyword. The header keyword
    #    is specified in the workflow parameter $combine_science (default obs.id). If only clustering on sky position
    #    is intended, then set $combine_science = 'instrumen'

    science_combination = (task("object_combination")
                           .with_recipe("muse_exp_combine")
                           .with_main_input(science)
                           .with_associated_input(filter_list, min_ret=0)
                           .with_associated_input(output_wcs, min_ret=0)
                           .with_associated_input(alignment, min_ret=1)
                           .with_job_processing(wavelength_range)
                           .with_input_filter(pixtable_reduced_class, offset_list_class)
                           .with_meta_targets([SCIENCE, IDP, IDP_DEEP])
                           .build())

    return science, science_combination
