from edps import QC0, task, subworkflow, FilterMode, IDP

from .muse_datasources import *
from .muse_task_functions import *

IDP_DEEP = "idp_deep"


# This subworkflow reduces and (eventually) combine sky exposures, as if they were regular science observations
@subworkflow("process_and_combine_sky", "")
def process_sky(response, lsf_calibrations, astrometry_calibrations, bias, dark, lamp_flat, wavelength,
                geometry_calibrations, sky_flat):
    # --- Pre-process sky raw exposures
    preprocess_sky = (task("preprocess_sky")
                      .with_recipe("muse_scibasic")
                      .with_main_input(raw_sky)
                      .with_associated_input(badpix_table, min_ret=0)
                      .with_associated_input(nonlinearity_gain, min_ret=0, condition=use_linearity_gain)
                      .with_associated_input(bias, [MASTER_BIAS])
                      .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                      .with_associated_input(lamp_flat, [TRACE_TABLE, MASTER_FLAT])
                      .with_associated_input(wavelength, [WAVECAL_TABLE])
                      .with_alternative_associated_inputs(geometry_calibrations)
                      .with_associated_input(sky_flat, [TWILIGHT_CUBE], min_ret=1, condition=use_skyflats)
                      .with_associated_input(raw_flat_illum, min_ret=0)
                      .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                      .with_meta_targets([QC0, IDP, IDP_DEEP])
                      .build())

    preprocess_sky_sci = (task("preprocess_sky_sci")
                          .with_recipe("muse_scibasic")
                          .with_main_input(raw_sky_sci)
                          .with_associated_input(badpix_table, min_ret=0)
                          .with_associated_input(nonlinearity_gain, min_ret=0, condition=use_linearity_gain)
                          .with_associated_input(bias, [MASTER_BIAS])
                          .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
                          .with_associated_input(lamp_flat, [TRACE_TABLE, MASTER_FLAT])
                          .with_associated_input(wavelength, [WAVECAL_TABLE])
                          .with_alternative_associated_inputs(geometry_calibrations)
                          .with_associated_input(sky_flat, [TWILIGHT_CUBE], min_ret=1, condition=use_skyflats)
                          .with_associated_input(raw_flat_illum, min_ret=0)
                          .with_input_filter(ARC_RED_LAMP, mode=FilterMode.REJECT)
                          .with_meta_targets([IDP])
                          .build())

    # --- Generate sky background to be subtracted
    sky = (task("sky")
           .with_recipe("muse_create_sky")
           .with_main_input(preprocess_sky)
           .with_associated_input(extinct_table)
           .with_associated_input(response, [STD_RESPONSE])
           .with_alternative_associated_inputs(lsf_calibrations)
           .with_associated_input(sky_lines)
           .with_associated_input(sky_continuum, min_ret=0)
           .with_associated_input(sky_mask, min_ret=0)
           .with_job_processing(wavelength_range)
           .with_meta_targets([QC0, IDP, IDP_DEEP])
           .build())

    # --- sky pixel tables are reduced
    science_sky = (task("object_sky")
                   .with_recipe("muse_scipost")
                   .with_main_input(preprocess_sky_sci)
                   .with_associated_input(extinct_table)
                   .with_associated_input(filter_list, min_ret=0)
                   .with_associated_input(raman_lines, min_ret=0)
                   .with_associated_input(offset_list, min_ret=0)
                   .with_associated_input(output_wcs, min_ret=0)
                   .with_alternative_associated_inputs(lsf_calibrations)
                   .with_associated_input(response, [STD_TELLURIC, STD_RESPONSE])
                   .with_associated_input(sky_lines)
                   .with_alternative_associated_inputs(astrometry_calibrations)
                   # Sky pixel tables are treated as object pixel table by overriding their TAG in the sof file.
                   .with_input_map({pixtable_sky_class: pixtable_object_class})
                   .with_grouping_keywords(['$combine_science', kwd.ins_mode])
                   .with_grouping_function(variable_clustering_by_sky_position)
                   .with_min_group_size(2)  # Combination of reduced exposures require at least 2 exposures
                   .with_job_processing(job_sky)
                   .with_meta_targets([IDP])
                   .build())

    # --- Alignment and combination of reduced exposures ---------------------------------------------------------------
    alignment_sky = (task("alignment_sky")
                     .with_recipe("muse_exp_align")
                     .with_main_input(science_sky)
                     .with_associated_input(source_list, min_ret=0)
                     .with_input_filter(image_fov_class)
                     .with_meta_targets([IDP])
                     .build())

    # The selection of sky exposures to combine is done in two steps.
    # 1. First, exposures are clustered by their sky coordinates. This step is regulated by two workflow parameters,
    #   "max_separation" and "max_diameter" . In order to include an exposure in a group of files for combination:
    #      - the exposure should be less distant than "max_separation" from the group.
    #      - the size of the group should be less than "max_diameter" after the inclusion of the new exposure.
    #
    # 2. Exposures in each cluster are grouped according to the value of a selected header keyword. The header keyword
    #    is specified in the workflow parameter $combine_science (default obs.id). If only clustering on sky position
    #    is intended, then set $combine_science = 'instrumen'
    sky_combination = (task("sky_combination")
                       .with_recipe("muse_exp_combine")
                       .with_main_input(science_sky)
                       .with_associated_input(filter_list, min_ret=0)
                       .with_associated_input(output_wcs, min_ret=0)
                       .with_associated_input(alignment_sky, min_ret=1)
                       .with_input_filter(pixtable_reduced_class, filter_list_class, offset_list_class)
                       .with_job_processing(wavelength_range)
                       .with_meta_targets([IDP])
                       .build())

    return sky, science_sky, sky_combination
