from edps import Job, get_parameter
from edps.generator.grouping import SkyPositionClusterGrouping
from edps.generator.parameters import Parameters, List, ClassifiedFitsFile, Dict

from . import muse_keywords as kwd


# --- Job processing functions ---------------------------------------
def job_object(job: Job):
    sky_source(job)
    wavelength_range(job)
    telluric_correction(job)
    autocalibration_science(job)


def job_sky(job: Job):
    telluric_correction(job)
    wavelength_range(job)
    autocalibration_sky(job)


def telluric_correction(job: Job):
    telluric = job.parameters.workflow_parameters['telluric_correction']

    if telluric != "TRUE":
        job.input_filter = [cat for cat in job.input_filter if cat != 'STD_TELLURIC']


def autocalibration_science(job: Job):
    autocalibration_strategy = job.parameters.workflow_parameters['autocalibration']
    if autocalibration_strategy in ["deepfield", "user", "none"]:
        job.parameters.recipe_parameters["muse.muse_scipost.autocalib"] = autocalibration_strategy
    elif autocalibration_strategy == "closest_sky":
        job.parameters.recipe_parameters["muse.muse_scipost.autocalib"] = "user"


def autocalibration_sky(job: Job):
    autocalibration_strategy = job.parameters.workflow_parameters['autocalibration']
    value = "deepfield" if autocalibration_strategy == "closest_sky" else "none"
    job.parameters.recipe_parameters["muse.muse_scipost.autocalib"] = value


# -----------------------------------------------------------------------------------------------------------------------

# --- WORKFLOW FUNCTIONS -----------------------------------------------------------------------

# This function handles the sky subtraction method, depending on the data reduction strategy selected by the
# skysubtraction parameter and the content of the dataset.
#
# skysubtraction = "auto" (default). If dedicated offset sky observations are present in the dataset, then they are used
#                   to evaluate and subtract the sky background contribution (both emission and continuum). Recipe
#                   parameters are set accordingly. If dedicated offset sky observations are not present, then static
#                   sky_lines calibrations are used but the intensity of emission lines is re-fitted on the target
#                   field of view. This method requires an input LSF_PROFILE.
# skysubtraction = "model". If dedicated offset sky observations are present, they are used to determine the sky
#                   continuum background. Sky emission lines are then re-fitted on the target field of view.
#                   If dedicated sky observations are not present, then static sky_lines calibrations are used but the
#                   intensity of emission lines is re-fitted on the target. This method requires an input LSF_PROFILE.
# skysubtraction = "subtract-model". To be used only if the dataset contains dedicated sky observations, which are used
#                   to evaluate and subtract the sky background contribution (both emission and continuum). Recipe
#                   parameters are set accordingly. This method requires an input LSF_PROFILE.
# skysubtraction = "simple". Simply sky-subtraction method is applied. It creates a sky spectrum from the science data,
#                   and directly subtracts it, without taking the LSF into account (LSF_PROFILE and input SKY files are
#                   ignored). It works on data that was not flux calibrated.
# skysubtraction = "none". No sky subtraction is performed.
def sky_source(job: Job):
    sky_method = job.parameters.workflow_parameters['skysubtraction']
    recipe_skymodel = f'muse.{job.command}.skymethod'
    recipe_skymodel_fraction = f'muse.{job.command}.skymodel_fraction'

    # Checks whether static sky_lines calibrations are present in the dataset. If present, it means that no dedicated
    # offset sky calibrations were present in the dataset, and therefore static calibrations were used.
    def is_static_skylines(f):
        return f.get_keyword_value(kwd.pro_catg, "None") == 'SKY_LINES' and \
            f.get_keyword_value(kwd.obs_name, "None") == 'STATIC'

    is_static_skylines_present = any([is_static_skylines(f) for f in job.associated_files])
    job.parameters.recipe_parameters[recipe_skymodel_fraction] = 0.4  # default

    # Set recipe parameters when using dedicated offset sky observations, without refitting emission lines.
    if sky_method == 'subtract_model' or (sky_method == 'auto' and not is_static_skylines_present):
        job.parameters.recipe_parameters[recipe_skymodel] = 'subtract-model'
        job.parameters.recipe_parameters[recipe_skymodel_fraction] = 0.75

    # Set recipe parameters when offset sky observations are not present, or (if present) the emission line contribution
    # is re-fitted on the target field of view.
    elif sky_method == 'model' or (sky_method == 'auto' and is_static_skylines_present):
        job.parameters.recipe_parameters[recipe_skymodel] = 'model'
        job.parameters.recipe_parameters[recipe_skymodel_fraction] = 0.2

    # Set recipe parameter when "simple" method is used
    elif sky_method == 'simple':
        job.parameters.recipe_parameters[recipe_skymodel] = 'simple'
        job.parameters.recipe_parameters[recipe_skymodel_fraction] = 0.2

    # Set recipe parameter if sky subtraction is not done
    elif sky_method == 'none':
        job.parameters.recipe_parameters[recipe_skymodel] = 'none'
        job.parameters.recipe_parameters[recipe_skymodel_fraction] = 0.2

    else:
        print(f"unknown sky_method='{sky_method}'")


# If telluric correction is not to be performed, then the STD_TELLURIC is removed from the dataset.
# This strategy is controlled by the "telluric_correction" workflow parameter.
# telluric_correction = "TRUE". Telluric correction is performed, othewise it is not


# Function to set the wavelength ranges in recipe parameters uniformely. Their values are set in the wavelength_min
# wavelength_min workflow parameters
def wavelength_range(job: Job):
    wavelength_min = job.parameters.workflow_parameters['wavelength_min']
    wavelength_max = job.parameters.workflow_parameters['wavelength_max']
    # The full name of the parameter depends on the recipe used in the task:
    recipe_lambda_min = f'muse.{job.command}.lambdamin'
    recipe_lambda_max = f'muse.{job.command}.lambdamax'
    job.parameters.recipe_parameters[recipe_lambda_min] = wavelength_min
    job.parameters.recipe_parameters[recipe_lambda_max] = wavelength_max


# --- Functions that determines the conditions for association or task execution ----------------------------------------

# Function that determines whether or not dark frames should be used in data reduction. The use of darks is generally not
# recommended, but it can be useful in those cases where the dark-current level is not homogeneous across the detector.
def use_darks(params):
    return get_parameter(params, 'use_darks') == 'yes'


def use_skyflats(params):
    return get_parameter(params, 'use_skyflats') == 'yes'


def use_autocalibration_user(params):
    return get_parameter(params, 'autocalibration') == 'user'


def use_autocalibration_sky(params):
    return get_parameter(params, 'autocalibration') == 'closest_sky'


# Function that determines whether or not linearity gain static calibrations should be used.
# The use of such static calibrations have not been yet assessed.
def use_linearity_gain(params):
    return get_parameter(params, 'use_linearity_gain') == 'yes'


# Function that determines whether or not the astrometric solution has to be re-computed using astrometric fields in the
# dataset. It is generally not recommended, but it can be useufl if the attached astrometric solution does not deliver
# the expected accuracy.
def recompute_astrometry(params):
    return get_parameter(params, 'recompute_astrometry') == 'yes'


# Function that determines whether or not the geometry table has to be re-computed using mask calibrations in the
# dataset. It is generally not recommended, but it can be useufl if the attached geometry table does not deliver the
# expected accuracy.
def recompute_geometry(params):
    return get_parameter(params, 'recompute_geometry') == 'yes'


# --- Selection of the line spread function calibration.
# The Line Spread Function calibrations to use are selected on the basis of user's choice
# by setting the lsfmode workflo parameter in muse_parameters.yaml
# lsfmode=any              Try to construct the LSF from dedicated lsf raw exposures. If not
#                          available, try to use arc raw exposures. If not available, use static
#                          calibration from the pipeline distribution.
# lsfmode=lsf              Try to construct the LSF from dedicated lsf raw exposures. If not
#                          available, use static calibration from the pipeline distribution.
# lsfmode=arc              Try to construct the LSF from dedicated arc raw exposures. If not
#                          available, use static calibration from the pipeline distribution.
# lsfmode=static (default) Use static calibration from the pipeline distribution.

def use_lsf_or_any(params):
    return get_parameter(params, 'lsfmode') in ('lsf', 'any')


def use_arc_or_any(params):
    return get_parameter(params, 'lsfmode') in ('arc', 'any')


# ---------------------------------------------------------------------------------------------------------------------
# User customized function to cluster exposures by sky coordinates, using clustering parameters
# that depends on input data.
# If input data are taken in NFM, the parameters used are max_diameter_NFM and max_separation_NFM
# If input data are taken in WFM, the parameters used are max_diameter_WFM and max_separation_WFM

Groups = Dict[str, List[ClassifiedFitsFile]]


def variable_clustering_by_sky_position(g: Groups, p: Parameters) -> Groups:
    if len(g) == 0:
        return {}
    else:
        result = {}
        for group_name, files_list in g.items():
            ins_mode = files_list[0].get_keyword_value('ins.mode', 'None')
            if "WFM" in ins_mode:
                max_d = "max_diameter_WFM"
                max_sep = "max_separation_WFM"
            elif "NFM" in ins_mode:
                max_d = "max_diameter_NFM"
                max_sep = "max_separation_NFM"
            else:
                max_d = None
                max_sep = None
            clustering = SkyPositionClusterGrouping("SKY.POSITION", max_diameter=max_d, max_separation=max_sep)
            res = clustering.split({group_name: files_list}, p)
            result = {**result, **res}
        return result
