from edps import QC1_CALIB, CALCHECKER
from edps import task, ReportInput

from .muse_astrometry import process_astrometry
from .muse_datasources import *
from .muse_line_spread_function import lsf
from .muse_monitoring_and_long_term_cals import monitoring
from .muse_response import process_standard
from .muse_science import process_science
from .muse_sky import process_sky
from .muse_task_functions import *

__title__ = "MUSE workflow"

# --- PROCESSING TASKS -------------------------------------------------------------------

# --- Process bias frames
bias = (task("bias")
        .with_recipe("muse_bias")
        .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
        .with_report("muse_master_bias", ReportInput.RECIPE_INPUTS_OUTPUTS)
        .with_main_input(raw_bias)
        .with_associated_input(badpix_table, min_ret=0)
        .with_meta_targets([QC1_CALIB])
        .build())

# --- Process dark frames
dark = (task("dark")
        .with_recipe("muse_dark")
        .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
        .with_report("muse_master_dark", ReportInput.RECIPE_INPUTS_OUTPUTS)
        .with_main_input(raw_dark)
        .with_associated_input(badpix_table, min_ret=0)
        .with_associated_input(bias, [MASTER_BIAS])
        .with_meta_targets([QC1_CALIB])
        .build())

# --- Process lamp flat frames
lamp_flat = (task("lamp_flat")
             .with_recipe("muse_flat")
             .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
             .with_report("muse_ifu_flat", ReportInput.RECIPE_INPUTS_OUTPUTS)
             .with_main_input(raw_flat_lamp)
             .with_associated_input(badpix_table, min_ret=0)
             .with_associated_input(bias, [MASTER_BIAS])
             .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
             .with_meta_targets([QC1_CALIB, CALCHECKER])
             .build())

# --- Process frames for wavelength calibrations
wavelength = (task("wavelength")
              .with_recipe("muse_wavecal")
              .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
              .with_report("muse_wave_cal", ReportInput.RECIPE_INPUTS_OUTPUTS)
              .with_main_input(raw_arc)
              .with_associated_input(badpix_table, min_ret=0)
              .with_associated_input(line_catalog)
              .with_associated_input(bias, [MASTER_BIAS])
              .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
              .with_associated_input(lamp_flat, [TRACE_TABLE])
              .with_meta_targets([QC1_CALIB, CALCHECKER])
              .build())

# --- Subworkflow to generate Line Spread Function calibration
lsf_calibrations, line_spread_function_from_lsf, line_spread_function_from_arcs = lsf(bias, dark, lamp_flat, wavelength)

# --- Subworkflow with monitoring tasks and geometry long term calibrations
linearity_and_gain, geometry_calibrations, geometry_calibrations_for_astrometry, throughput = monitoring(bias, dark,
                                                                                                         lamp_flat,
                                                                                                         wavelength)

# --- Process sky flats raw calibrations
sky_flat = (task("sky_flat")
            .with_recipe("muse_twilight")
            .with_report("muse_rawdisp", ReportInput.RECIPE_INPUTS)
            .with_report("muse_sky_flat", ReportInput.RECIPE_INPUTS_OUTPUTS)
            .with_main_input(raw_flat_sky)
            .with_associated_input(badpix_table, min_ret=0)
            .with_associated_input(bias, [MASTER_BIAS])
            .with_associated_input(dark, [MASTER_DARK], min_ret=0, condition=use_darks)
            .with_associated_input(lamp_flat, [MASTER_FLAT, TRACE_TABLE])
            .with_associated_input(wavelength, [WAVECAL_TABLE])
            .with_alternative_associated_inputs(geometry_calibrations)
            .with_associated_input(vignetting_mask, min_ret=0)
            .with_associated_input(raw_flat_illum, min_ret=0)
            .with_meta_targets([QC1_CALIB, CALCHECKER])
            .build())

# --- Subworkflow to generate response curve and telluric correction
response = process_standard(bias, dark, lamp_flat, wavelength, geometry_calibrations, sky_flat)

# --- Subworkflow to process astrometric calibrations
astrometry, astrometry_calibrations = process_astrometry(response, bias, dark, lamp_flat, wavelength,
                                                         geometry_calibrations_for_astrometry, sky_flat)

# --- Subworkflow to reduce and combine offset sky exposures as if they were regular science observations
sky, science_sky, sky_combination = process_sky(response, lsf_calibrations, astrometry_calibrations, bias, dark,
                                                lamp_flat, wavelength, geometry_calibrations, sky_flat)

# --- Subworkflow to reduce and combine science observations
science, science_combination = process_science(sky, response, lsf_calibrations, astrometry_calibrations, bias, dark,
                                               lamp_flat, wavelength, geometry_calibrations, sky_flat, science_sky)
